#include <ros/ros.h>

#include <interactive_markers/interactive_marker_server.h>
#include <interactive_markers/menu_handler.h>
#include <visualization_msgs/MarkerArray.h>
#include <geometry_msgs/Point.h>

#include <tf/transform_broadcaster.h>
#include <tf/tf.h>

#include <math.h>

using namespace visualization_msgs;

ros::Publisher* goalOutPubPtr = nullptr;
boost::shared_ptr<interactive_markers::InteractiveMarkerServer> server;
ros::Publisher* pub_marko;

Marker makeArrow( float size ){
  	Marker marker;
  	marker.type = Marker::ARROW;
  	marker.scale.x = 2*size;
  	marker.scale.y = size;
  	marker.scale.z = size;
  	marker.color.r = 0.2;
  	marker.color.g = 0.2;
 	marker.color.b = 0.8;
  	marker.color.a = 1.0;  
  	return marker;
}

void processFeedback( const visualization_msgs::InteractiveMarkerFeedbackConstPtr &feedback ){
  server->applyChanges();
}

void alignMarker( const visualization_msgs::InteractiveMarkerFeedbackConstPtr &feedback ){
  	geometry_msgs::Pose pose = feedback->pose;
  	ros::Duration(0.2).sleep();
  	pose.position.z = pose.position.z+2;
	server->setPose( feedback->marker_name, pose );
  	server->applyChanges();
  	ros::Duration(0.2).sleep();
  	pose.position.z = pose.position.z-3;
	server->setPose( feedback->marker_name, pose );
  	server->applyChanges();
	ros::Duration(0.2).sleep();
  	pose.position.z = pose.position.z+1;
	server->setPose( feedback->marker_name, pose );
  	server->applyChanges();
  	geometry_msgs::Point msg;
  	msg.x = pose.position.x;
  	msg.y = pose.position.y;
  	msg.z = pose.position.z-0.4;
  	if(goalOutPubPtr!=nullptr){
  		goalOutPubPtr->publish(msg);
  	}
}

double rand( double min, double max ){
  double t = (double)rand() / (double)RAND_MAX;
  return min + t*(max-min);
}

void makeChessPieceMarker( const tf::Vector3& position ){
  InteractiveMarker int_marker;
  int_marker.header.frame_id = "map";
  tf::pointTFToMsg(position, int_marker.pose.position);
  int_marker.scale = 1;
  int_marker.pose.orientation.w = 1;
  int_marker.pose.orientation.x = 0;
  int_marker.pose.orientation.y = 1;
  int_marker.pose.orientation.z = 0;
  int_marker.pose.position.z = 0.4;
  int_marker.name = "chess_piece";
  int_marker.description = "Set goal";
  InteractiveMarkerControl control;
  control.orientation.w = 1;
  control.orientation.x = 0;
  control.orientation.y = 0;
  control.orientation.z = 0;
  control.interaction_mode = InteractiveMarkerControl::MOVE_PLANE;
  int_marker.controls.push_back(control);
  control.markers.push_back( makeArrow( 0.3 ) );
  control.always_visible = true;
  int_marker.controls.push_back(control);
  server->insert(int_marker);
  server->setCallback(int_marker.name, &processFeedback);
  server->setCallback(int_marker.name, &alignMarker, visualization_msgs::InteractiveMarkerFeedback::MOUSE_UP );
}


int main(int argc, char** argv){
  ros::init(argc, argv, "basic_controls");
  ros::NodeHandle nh; 
  server.reset( new interactive_markers::InteractiveMarkerServer("basic_controls","",false) );
  ros::Duration(0.1).sleep();
  ros::Publisher goalOutPub = nh.advertise<geometry_msgs::Point> ( "goal" ,2);
  goalOutPubPtr = &goalOutPub;
  tf::Vector3 position = tf::Vector3( 0, 0, 0);
  makeChessPieceMarker( position );
  server->applyChanges();
  ros::spin();
  server.reset();
}


